/**
 * Copyright 2008-2012 Charles Amey
 *
 * This file is part of Fuel Manager.
 *
 * Fuel Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Fuel Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fuel Manager.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QSettings>
#include <QSqlQuery>
#include <QSqlError>
#include <QVector>
#include <QDebug>
#include <QtGui>
#include <QtSql>
#include <QIcon>

#include "about.h"
#include "alldata.h"
#include "configdialog.h"
#include "entryform.h"
#include "login.h"
#include "mainwindow.h"
#include "selectvehicledialog.h"
#include "statistics.h"
#include "assistant.h"

MainWindow::MainWindow(QMainWindow *parent)
	:QMainWindow(parent)
{
	mw.setupUi(this);
	
	//set the icon
	QString filename = "/usr/share/icons/hicolor/scalable/apps/fuelmanager.svg";
	
	QFileInfo iconFilename(filename);
	if (iconFilename.exists())
	{
		QIcon icon(filename);
		setWindowIcon(icon);
	}
	
	//set up the basic info for qsettings
	QCoreApplication::setOrganizationName("fuelmanager");
	//QCoreApplication::setOrganizationDomain("homelinux.com");
	QCoreApplication::setApplicationName("fuelmanager");

	username = "";
	password = "";
	alreadyLoggedIn = false;
	//check the settings
	checkSettings();
	
	//link up the about qt button
	connect(mw.actionAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	
	//link up the help button
	connect(mw.actionHelp,SIGNAL(triggered()), this, SLOT(help()));
	
	assistant = new Assistant;
	
	
}//end constructor

//destructor
MainWindow::~MainWindow()
{
	//because it spawns a qprocess, we gotta kill it
	delete assistant;
}

//read the config file
void MainWindow::checkSettings()
{
	QSettings settings;
	QString databaseType = settings.value("config/databasetype").toString();
	QString savedVehicleId = settings.value("config/vehicle").toString();
	//check to see which database is selected
	if (databaseType == "")
	{
		//qDebug("no database in the config file, show config dialog");
		configureFuelManager();
	}
	if (databaseType == "sqlite")
	{
		//disable the login menu option
		mw.actionLogin->setEnabled(false);
		sqliteDriver();
	}	//check and make sure the sqlite driver is available
	if (databaseType == "mysql")
	{
		mw.actionLogin->setEnabled(true);
		mysqlDriver();
	}
	
	//check for a saved vehicle
	if ( savedVehicleId != "")
	{
		if (conn.isOpen())
		{
			QSqlQuery query(conn);
			query.prepare("select id,description from vehicles where id = :vehicleid");
			query.bindValue(":vehicleid",savedVehicleId);
			query.exec();
			query.next();
			record = query.record();
			showEverything();
		}
		else
		{
			QMessageBox mBox (QMessageBox::Critical, "Not logged in",
				"Please login first!  Click file->login.", QMessageBox::Ok, this,Qt::Dialog);
			mBox.exec();
		}
	}
}//end checkSettings

//configuration 
void MainWindow::configureFuelManager()
{
	//qDebug("show the configure dialog box");
	ConfigDialog configure(this);
	if (configure.exec())
	{
		//re-read the configuration file
		checkSettings();
	}
// 	else
// 	{
// 		//checkSettings();
// 		qDebug()<<"they just closed the configure dialog";
// 	}
} //end configuration

void MainWindow::showEverything()
{
	//remove all the tabs first
	mw.tabWidget->clear();
	if (conn.isOpen())
	{
		//set the vehicle description
		QString desc = record.value("description").toString();
		mw.vehicleLabel->setText(desc);

		//add the entry tab
		EntryForm *myEntryForm = new EntryForm(this);
		mw.tabWidget->addTab(myEntryForm,myEntryForm->windowTitle());
		
		//add the all data tab
		everything = new AllData(this);
		mw.tabWidget->addTab(everything, everything->windowTitle());

		//add the statistics tab
		stats = new Statistics(this);
		mw.tabWidget->addTab(stats,stats->windowTitle());
	}
	else
	{
		QMessageBox mBox (QMessageBox::Critical, "Not logged in",
		"Please login first!  Click file->login.", QMessageBox::Ok, this,Qt::Dialog);
		mBox.exec();
	}
	
// 	//set the vehicle description
// 	QString desc = record.value("description").toString();
// 	mw.vehicleLabel->setText(desc);
// 
// 	//add the entry tab
// 	EntryForm *myEntryForm = new EntryForm(this);
// 	mw.tabWidget->addTab(myEntryForm,myEntryForm->windowTitle());
// 	
// 	//add the all data tab
// 	everything = new AllData(this);
// 	mw.tabWidget->addTab(everything, everything->windowTitle());
// 
// 	//add the statistics tab
// 	stats = new Statistics(this);
// 	mw.tabWidget->addTab(stats,stats->windowTitle());

}//end showEverything

QString MainWindow::getConnectionName()
{
	return conn.connectionName();
}

QSqlRecord MainWindow::getRecord()
{
	return record;
}

//show the about dialog
void MainWindow::about()
{
	About about;
	about.exec();
}

void MainWindow::help()
{
	assistant->showDocs("index.html");
}

void MainWindow::sqliteDriver()
{
	QSettings settings;
	QString dbaseFileName = settings.value("config/filename").toString();
	QString dbaseLocation = settings.value("config/location").toString();
	QString connectionName = settings.value("config/connection").toString();
	QString db = dbaseLocation + dbaseFileName;
	if (! QSqlDatabase::contains(connectionName))
	{
		QSqlDatabase::addDatabase("QSQLITE",connectionName);
	}
	conn = QSqlDatabase::database(connectionName,false);
	conn.setDatabaseName(db);
	if (conn.open())
	{
		//create vehicles
		QSqlQuery query(conn);
		QString q = "create table if not exists vehicles (id integer primary key on conflict fail autoincrement, ";
		q += "description text )";
		query.exec(q);
		query.finish();
		//create fuel_mileage table
		q = "create table if not exists fuel_mileage (id integer primary key on conflict fail autoincrement, ";
		q += "miles real,";
		q += "gallons real, ";
		q += "cost real, ";
		q += "fillup_date text, ";
		q += "vehicle_id integer not null, ";
		q += "foreign key (vehicle_id) references vehicles (id) ";
		q += ")";
		query.exec(q);
		query.finish();
		//turn on foreign keys
		q = "pragma foreign_keys = on";
		query.exec(q);
		query.finish();
	}
	//qDebug()<<"after opening: "<<conn.lastError();

}//end sqliteDriver

void MainWindow::mysqlDriver()
{
	QSettings settings;
	conn.setHostName(settings.value("config/hostname").toString());
	conn.setDatabaseName(settings.value("config/dbasename").toString());
	conn.setPort(settings.value("config/port").toInt());
	QString connectionName = settings.value("config/connection").toString();
	if (! QSqlDatabase::contains(connectionName))
	{
		QSqlDatabase::addDatabase("QMYSQL",connectionName);
	}
	conn = QSqlDatabase::database(connectionName,false);
	if (!alreadyLoggedIn)
	{
		login();
	}
	if (conn.open(username,password))
	{
		QSqlQuery query(conn);
		//create the vehicles table
		QString q = "create table if not exists vehicles (";
		q += "id int(11) not null auto_increment, ";
		q += "description varchar(100) default null,";
		q += "primary key (id) ";
		q += ") engine=innodb default charset=latin1 ";
		query.exec(q);
		query.finish();
		
		//create the fuel_mileage table
		q = "create table if not exists fuel_mileage (";
		q += "id int(11) not null auto_increment, ";
		q += "miles float default null, ";
		q += "gallons float default null, ";
		q += "cost float default null, ";
		q += "fillup_date date default null, ";
		q += "vehicle_id int(11) default null, ";
		q += "primary key (id), ";
		q += "key vehicle_id_fk (vehicle_id), ";
		q += "constraint vehicle_id_fk foreign key (vehicle_id) ";
		q += "references vehicles (id) on delete no action on update no action )";
		q += "engine=innodb default charset=latin1";
		query.exec(q);
		query.finish();
	}//end connection is open
	
}//end mysqlDriver

//login dialog box
bool MainWindow:: login ()
{
	bool loginOk = false;
	
	QSettings settings;
	//read the config file
	conn.setHostName(settings.value("config/hostname").toString());
	conn.setDatabaseName(settings.value("config/dbasename").toString());
	conn.setPort(settings.value("config/port").toInt());
	Login mylogin;
	if (mylogin.exec())
	{
		username = mylogin.getUsername();
		password = mylogin.getPassword();
		//open the mysql dbase connection
		if (conn.open(username,password))
		{
			loginOk = true;
			alreadyLoggedIn = true;
		}
		else
		{
			loginOk = false;
			QMessageBox::warning(this,"Login Failed!", conn.lastError().text(),QMessageBox::Ok,0,0);
		}
	}
	else
	{
		loginOk = false;
	}
	return loginOk;
} //end login

void MainWindow::selectVehicle()
{
	//qDebug()<<"select vehicle dialog box goes here";
	//check and see if the database is open first
	if (conn.isOpen())
	{
		QSettings settings;
		SelectVehicleDialog choose(this);
		if (choose.exec())
		{
			record = choose.getRecord();
			settings.setValue("config/vehicle", record.value("id").toInt());
			showEverything();
		}
	}
	else
	{
		QMessageBox mBox (QMessageBox::Critical, "Not logged in",
		"Please login first!  Click file->login.", QMessageBox::Ok, this,Qt::Dialog);
		
		
// 		QMessageBox msgBox;
// 		QString text = "Login to your database first!\n";
// 		text += "Click file->login.";
// 		msgBox.setText(text);
		
		mBox.exec();
		//qDebug()<<"dbase is NOT open!!, show dialog saying so";
	}
		
	
	
// 	QSettings settings;
// 	SelectVehicleDialog choose(this);
// 	if (choose.exec())
// 	{
// 		record = choose.getRecord();
// 		settings.setValue("config/vehicle", record.value("id").toInt());
// 		showEverything();
// 	}
}//end selectVehicle

//when tabs get clicked on, update the info on that particular tab
void MainWindow::updateAllData()
{
	if (mw.tabWidget->tabText(mw.tabWidget->currentIndex()) == "All Data")
	{
		everything->refreshTable();
	}
	if (mw.tabWidget->tabText(mw.tabWidget->currentIndex()) == "Statistics")
	{
		stats->refreshTables();
	}
} //end updateAllData

void MainWindow::exportData()
{
	//qDebug()<<"export the data for the currently selected vehicle";
	if (conn.isOpen())
	{
		QFileDialog dialog(this, tr("Save As..."));
		dialog.setFileMode(QFileDialog::AnyFile);
		dialog.setViewMode(QFileDialog::Detail);
		dialog.setDirectory(QDir::homePath()+"/");
		QStringList filename;
		if (dialog.exec())
		{
			filename = dialog.selectedFiles();
			if (filename[0] != "")
			{
				//qDebug()<<filename[0];
				if (QFile::exists(filename[0]))
				{
					QMessageBox message;
					message.setText(tr("File already exists!"));
					message.setInformativeText(tr("Overwrite?"));
					message.setStandardButtons(QMessageBox::Yes|QMessageBox::No);
					message.setDefaultButton(QMessageBox::No);
					//int value = message.exec();
					if (message.exec() == QMessageBox::Yes)
					{
						//qDebug()<<"overwrite the file and write out the data";
						writeData(filename[0]);
					}
				}//if file exists
				else
				{
					//qDebug()<<"create file and write out the data";
					writeData(filename[0]);
				}
			}
		}//end dialog executed		
	}
	else
	{
		QMessageBox mBox (QMessageBox::Critical, "Not logged in",
		"Please login first!  Click file->login.", QMessageBox::Ok, this,Qt::Dialog);
		mBox.exec();
	}

}//end exportData();

void MainWindow::writeData(QString f)
{
	//create file to write to
	QFile file(f);
	file.open(QIODevice::WriteOnly|QIODevice::Text);
	QTextStream out(&file);	

	QSqlRecord r = this->getRecord();
	int vehicleId = r.value("id").toInt();

	QSqlQuery query(conn);
	query.prepare("select miles,gallons,cost,fillup_date from fuel_mileage \
		where vehicle_id = :vehicleid");
	query.bindValue(":vehicleid",vehicleId);
	if (query.exec())
	{
		out<<"\"miles\",\"gallons\",\"cost\",\"fillup_date\""<<"\n";
		while(query.next())
		{
			QSqlRecord t = query.record();
			QString theData = "";
			for (int i=0;i<t.count();i++)
			{
				theData = theData + "\"" + t.value(i).toString() + "\",";
			}
			theData = theData.remove(theData.length()-1,1);
			//qDebug()<<theData;
			out<<theData<<"\n";
		}//loop through resultset
		file.close();
		QMessageBox mBox (QMessageBox::Information, "Export Data",
			"Export is completed!", QMessageBox::Ok, this,Qt::Dialog);
		mBox.exec();
		
	}//query was executed
	
	//QSqlDatabase c = QSqlDatabase::database(this->getConnectionName());
	//qDebug()<<"writeData for id: "<<vehicleId;
// 	if (c.isOpen())
// 	{
// 		QSqlQuery query(c);
/*		query.prepare("select miles,gallons,cost,fillup_date from fuel_mileage \
			where vehicle_id = :vehicleid");
*/			
// 		query.bindValue(":vehicleid",vehicleId);
// 		if (query.exec())
// 		{
// 			out<<"\"miles\",\"gallons\",\"cost\",\"fillup_date\""<<"\n";
// 			while(query.next())
// 			{
// 				QSqlRecord t = query.record();
// 				QString theData = "";
// 				for (int i=0;i<t.count();i++)
// 				{
// 					theData = theData + "\"" + t.value(i).toString() + "\",";
// 				}
// 				theData = theData.remove(theData.length()-1,1);
// 				//qDebug()<<theData;
// 				out<<theData<<"\n";
// 			}//loop through resultset
// 			file.close();
// 		}//query was executed
// 	}//database connection is open
}//end writeData

QSqlDatabase MainWindow::getDatabase()
{
	return conn;
}

