/**
 * Copyright 2008-2012 Charles Amey
 * 
 * This file is part of Fuel Manager.
 * 
 * Fuel Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Fuel Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Fuel Manager.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QTableWidget>
#include <QVBoxLayout>
#include <QSettings>
#include <QSqlQuery>
#include <QMessageBox>
#include <QDebug>
#include <QSqlError>
#include "statistics.h"
#include "monthmodel.h"

Statistics::Statistics(MainWindow *p)
	: QWidget(p)
{
	statsBase.setupUi(this);
	parent = p;
	columnNames += "Miles";
	columnNames += "Gallons";
	columnNames += "MPG";
	columnNames += "Cost";

	themonths+="January";
	themonths+="February";
	themonths+="March";
	themonths+="April";
	themonths+="May";
	themonths+="June";
	themonths+="July";
	themonths+="August";
	themonths+="September";
	themonths+="October";
	themonths+="November";
	themonths+="December";

	//get rid of all the tabs
	statsBase.tabWidget->clear();

	//create the lifetime tab
	QWidget *lifetime = new QWidget;
	lifetime->setWindowTitle("Lifetime");

	//create and setup a new table
	lifetimeTable = new QTableWidget;
	lifetimeTable->setColumnCount(columnNames.size());
	lifetimeTable->setHorizontalHeaderLabels(columnNames);

	//creat a new vertical box layout and add the table 
	QVBoxLayout *lifetimeLayout = new QVBoxLayout;
	lifetimeLayout->addWidget(lifetimeTable);
	lifetime->setLayout(lifetimeLayout);

	//add the lifetime tab
	statsBase.tabWidget->addTab(lifetime,lifetime->windowTitle());

	//add the yearly tab
	QWidget *yearly = new QWidget;
	yearly->setWindowTitle("Yearly");
	yearlyTable = new QTableWidget;
	QVBoxLayout *yearlyLayout = new QVBoxLayout;
	yearlyLayout->addWidget(yearlyTable);
	yearly->setLayout(yearlyLayout);
	statsBase.tabWidget->addTab(yearly,yearly->windowTitle());

	//add the monthly tab
	QWidget *monthly = new QWidget;
	monthly->setWindowTitle("Monthly");
	monthlyTable = new QTableView;
	QVBoxLayout *monthlyLayout = new QVBoxLayout;
	monthlyLayout->addWidget(monthlyTable);
	monthly->setLayout(monthlyLayout);
	statsBase.tabWidget->addTab(monthly,monthly->windowTitle());
	
	model = new MonthModel;
	
} //end constructor

//destructor
Statistics::~Statistics()
{
	delete model;
}

//lifetime statistics
void Statistics::lifetimeStats ()
{
	QSqlDatabase c = QSqlDatabase::database(parent->getConnectionName());
	
	
	QSqlRecord r = parent->getRecord();
	int vehicleId = r.value("id").toInt();

	//get the locale's symbol instead of hard-coding a $
	QString currencySymbol = QLocale().currencySymbol();

	//clear the table first
	lifetimeTable->clear ();
	double miles=0.0,gallons=0.0,mpg = 0.0, cost=0.0;
	QString formattedString;
	QTableWidgetItem *temp;

	QFont lifetimeStatsFont;
	lifetimeStatsFont.setPointSize(16);

	if (c.isOpen())
	{
		QSqlQuery query(c);
		query.prepare("select miles,gallons,cost from fuel_mileage where vehicle_id = :vehicleid");
		query.bindValue(":vehicleid",vehicleId);
		if (query.exec())
		{
			while (query.next())
			{
				//keep running sums
				miles = miles + query.value(0).toDouble(); //miles
				gallons = gallons + query.value(1).toDouble(); //gallons
				cost = cost + query.value(2).toDouble (); //cost
			}
			if (miles != 0.0)
				mpg = miles/gallons;
			//set the number of rows
			lifetimeTable->setRowCount(1);
			//set the number of columns
			lifetimeTable->setColumnCount(columnNames.size());
			//set column names
			lifetimeTable->setHorizontalHeaderLabels(columnNames);

			formattedString = QString("%L1").arg(miles,0,'f',2);
			//temp = new QTableWidgetItem(QString::number(miles,'f',2));
			temp = new QTableWidgetItem(formattedString);
			temp->setFlags(Qt::ItemIsEnabled);
			//temp->setFont(lifetimeStatsFont);
			lifetimeTable->setItem(0,0,temp);

			//QString str = QString("%L1").arg(d,0,'f',2);
			//temp = new QTableWidgetItem(str);
			formattedString = QString("%L1").arg(gallons,0,'f',2);
			//temp = new QTableWidgetItem(QString::number(gallons,'f',2));
			temp = new QTableWidgetItem(formattedString);
			temp->setFlags(Qt::ItemIsEnabled);
			//temp->setFont(lifetimeStatsFont);
			lifetimeTable->setItem(0,1,temp);

			formattedString = QString("%L1").arg(mpg,0,'f',2);
			//temp = new QTableWidgetItem(QString::number(mpg,'f',2));
			temp = new QTableWidgetItem(formattedString);
			temp->setFlags(Qt::ItemIsEnabled);
			//temp->setFont(lifetimeStatsFont);
			lifetimeTable->setItem(0,2,temp);

			formattedString = QString(currencySymbol+"%L1").arg(cost,0,'f',2);
			//qDebug()<<formattedString;
			//QString t = "$" + QString::number(cost,'f',2);
			temp = new QTableWidgetItem(formattedString);
			temp->setFlags(Qt::ItemIsEnabled);
			//temp->setFont(lifetimeStatsFont);
			lifetimeTable->setItem(0,3,temp);

			//resize rows and columns
			lifetimeTable->resizeColumnsToContents();
			lifetimeTable->resizeRowsToContents();
		}
	} //connection is open
	else
	{
		//good error message to indicate that the database connection is not open
		QMessageBox mBox (QMessageBox::Critical, "Statistics Problem!", 
			"Connection is not open!  Please login first.", QMessageBox::Ok, this,Qt::Dialog);
		mBox.exec();
	}
} //end lifetime stats

//stats for each year
void Statistics::yearlyStats()
{
	QSqlDatabase c = QSqlDatabase::database(parent->getConnectionName());
	QSqlRecord r = parent->getRecord();
	int vehicleId = r.value("id").toInt();
	
	//get the locale's symbol instead of hard-coding a $
	QString currencySymbol = QLocale().currencySymbol();

	yearlyTable->clear();
	double miles=0.0,gallons=0.0,mpg = 0.0, cost=0.0;
	QString formattedString;
	QTableWidgetItem *temp;

	QFont yearlyStatsFont;
	yearlyStatsFont.setPointSize(14);

	QSettings settings;
	QString dbase = settings.value("config/databasetype").toString();
	//make sure the connection is open first
	if (c.isOpen())
	{
		QSqlQuery query(c);
		if (dbase == "sqlite")
		{
			query.prepare("select distinct strftime(\"%Y\",fillup_date) from fuel_mileage \
				where strftime(\"%Y\",fillup_date) != '0000' \
				and vehicle_id = :vehicleid"
			);
		}
		if (dbase == "mysql")
		{
			query.prepare("SELECT distinct(date_format(fillup_date,'%Y')) \
			FROM fuel_mileage \
			where date_format(fillup_date,'%Y') != '0000' \
			and vehicle_id = :vehicleid \
			");
		}
		query.bindValue(":vehicleid", vehicleId);
		if(query.exec())
		{
			//get the row names first
			QStringList rowNames;
			while (query.next())
			{
				rowNames += query.value(0).toString();
			}
			//set up the row and column count
			yearlyTable->setRowCount(rowNames.size());
			yearlyTable->setColumnCount(columnNames.size());
			//set the row and column names
			yearlyTable->setVerticalHeaderLabels(rowNames);
			yearlyTable->setHorizontalHeaderLabels(columnNames);
			yearlyTable->setAlternatingRowColors(true);
			//loop through the row names
			for(int i=0;i<rowNames.size();i++)
			{
				QString e = "";
				if (dbase == "sqlite")
				{
					e = "select miles,gallons,cost from fuel_mileage ";
					e = e + "where strftime(\"%Y\",fillup_date) = \"" + rowNames.value(i)+"\"";
					
					query.prepare("select miles,gallons,cost from fuel_mileage \
					where strftime(\"%Y\",fillup_date) = :fillup_date \
					and vehicle_id = :vehicleid \
					");
				}
				if (dbase == "mysql")
				{
					e = "select miles,gallons,cost from fuel_mileage ";
					e = e + "where date_format(fillup_date,'%Y') = '" + rowNames.value(i) + "\'";
					
					query.prepare("select miles,gallons,cost from fuel_mileage \
					where date_format(fillup_date,'%Y') = :fillup_date \
					and vehicle_id = :vehicleid \
					");
				}
				query.bindValue(":fillup_date", rowNames.value(i));
				query.bindValue(":vehicleid", vehicleId);
				//qDebug()<<e;
				if (query.exec())
				{
					while (query.next())
					{
						miles = miles + query.value(0).toDouble();
						gallons = gallons + query.value(1).toDouble();
						cost = cost + query.value(2).toDouble();
					}
					if (miles != 0.0)
						mpg = miles/gallons;
					else
						mpg = 0.0;
					//write the current stuff to the table
					formattedString = QString("%L1").arg(miles,0,'f',2);
					temp = new QTableWidgetItem(formattedString);
					temp->setFlags(Qt::ItemIsEnabled);
					//temp->setFont(yearlyStatsFont);
					//temp = new QTableWidgetItem(QString::number(miles,'f',2));
					yearlyTable->setItem(i,0,temp);

					formattedString = QString("%L1").arg(gallons,0,'f',2);
					temp = new QTableWidgetItem(formattedString);
					temp->setFlags(Qt::ItemIsEnabled);
					//temp->setFont(yearlyStatsFont);
					//temp = new QTableWidgetItem(QString::number(gallons,'f',2));
					yearlyTable->setItem(i,1,temp);

					formattedString = QString("%L1").arg(mpg,0,'f',2);
					temp = new QTableWidgetItem(formattedString);
					temp->setFlags(Qt::ItemIsEnabled);
					//temp->setFont(yearlyStatsFont);
					//temp = new QTableWidgetItem(QString::number(mpg,'f',2));
					yearlyTable->setItem(i,2,temp);
					
					formattedString = QString(currencySymbol+"%L1").arg(cost,0,'f',2);
					temp = new QTableWidgetItem(formattedString);
					temp->setFlags(Qt::ItemIsEnabled);
					//temp->setFont(yearlyStatsFont);
					//temp = new QTableWidgetItem("$"+QString::number(cost,'f',2));
					yearlyTable->setItem(i,3,temp);

					//resize both rows and columns
					yearlyTable->resizeRowsToContents();
					yearlyTable->resizeColumnsToContents();
				}
				else
				{
					QMessageBox::critical(this,tr("yearly Error!"),query.lastError().text(),QMessageBox::Ok,0,0);
				}
				//reset the totals to start the next iteration
				miles = 0.0; gallons = 0.0; mpg = 0.0; cost=0.0;
			}//end for
		}//end big query
		else
		{
			QMessageBox::critical(this,tr("Error!"),query.lastError().text(),QMessageBox::Ok,0,0);
		}
	} //connection was open
	else
	{
		//message box if the connection is not open
		QMessageBox mBox (QMessageBox::Critical, "stats Problem!", 
			"Connection is not open!  Please login first.", QMessageBox::Ok, this,Qt::Dialog);
		mBox.exec();
	}
} //end yearly stats

//monthly statistics
void Statistics::monthlyStats()
{
	QSqlDatabase c = QSqlDatabase::database(parent->getConnectionName());
	QSqlRecord r = parent->getRecord();
	int vehicleId = r.value("id").toInt();

	//get the locale's symbol instead of hard-coding a $
	QString currencySymbol = QLocale().currencySymbol();
	
	//clear out the model
	model->clearEverything();
	
	//set up the colors to rotate by
	//QColor one = Qt::lightGray;
	QColor one(224,229,233);
	QColor two = Qt::white;
	
	double miles=0.0,gallons=0.0,mpg = 0.0, cost=0.0;
	int fillups = 0;
	
	//which color
	int whichcolor = 0;
	
	QSettings settings;
	QString dbase = settings.value("config/databasetype").toString();
	//make sure the connection is open!
	if (c.isOpen())
	{
		QSqlQuery query(c);
		if (dbase == "sqlite")
		{
			query.prepare("select distinct strftime(\"%Y\",fillup_date) from fuel_mileage \
			where strftime(\"%Y\",fillup_date) != '0000' \
			and vehicle_id = :vehicleid \
			");
		}
		if (dbase == "mysql")
		{
			query.prepare("SELECT distinct(date_format(fillup_date,'%Y')) \
			FROM fuel_mileage where date_format(fillup_date,'%Y') != '0000' \
			and vehicle_id = :vehicleid \
			");
		}
		query.bindValue(":vehicleid", vehicleId);
		QStringList yearnames;
		if(query.exec())
		{
			int i=0;
			while (query.next())
			{
				yearnames += query.value(0).toString();
				//set up the column names
				model->setHeaderData(i,Qt::Horizontal,query.value(0).toString(),Qt::DisplayRole);
				//model->setHeaderData(i,Qt::Horizontal,query.value(0).toString());
				i++;
			}
			//set up the row names (vertical)
			for (int i = 0; i < themonths.size(); ++i)
			{
				model->setHeaderData(i,Qt::Vertical,themonths[i],Qt::DisplayRole);
			}

			//get all the data
			for (int i=0;i<themonths.size();i++)
			{
				//colors for the entire row
				QList <QColor> rowcolor;
				//empty the qlist
				rowcolor.clear();
				//list of data for the entire row
				QList<QVariant> t;
				//empty out the list
				t.clear();
				
				for(int j=0;j<yearnames.size();j++)
				{
					//qDebug()<<"i:"<<i<<" j:"<<j;
					//reset the totals to start with
					miles=0.0;gallons=0.0;mpg = 0.0;cost=0.0;
					fillups=0;
					int mn = i+1;
					QString monthnumber = "";
					if (mn <10)
					{
						monthnumber = monthnumber + '0' + QString::number(mn);
					}
					else
					{
						monthnumber = QString::number(mn);
					}
					//qDebug()<< monthnumber;
					if (dbase == "sqlite")
					{
						query.prepare("select miles,gallons,cost from fuel_mileage \
						where strftime(\"%Y\",fillup_date) = :fillup_year \
						and strftime(\"%m\",fillup_date) = :fillup_month \
						and vehicle_id = :vehicleid \
						");
					}
					if (dbase == "mysql")
					{
						query.prepare("select miles,gallons,cost from fuel_mileage \
						where date_format(fillup_date,'%Y') = :fillup_year \
						and date_format(fillup_date,'%m') = :fillup_month \
						and vehicle_id = :vehicleid \
						");
					}
					query.bindValue(":fillup_year",yearnames.value(j));
					query.bindValue(":fillup_month",monthnumber);					
					query.bindValue(":vehicleid", vehicleId);
					//qDebug()<<q;
					if (query.exec())
					{
						while (query.next())
						{
							miles = miles + query.value(0).toDouble(); //miles
							gallons = gallons + query.value(1).toDouble(); //gallons
							cost = cost + query.value(2).toDouble (); //cost
							fillups++;
						}
						if (gallons != 0)
							mpg = miles/gallons;
						else
							mpg = 0.0;
						QString string = "\n";
						if (fillups == 0)
						{
							t.append(string);
							if (whichcolor == 0)
								rowcolor.append(one);
							else
								rowcolor.append(two);
						}
						else
						{
							string = "Fillups: " + QString::number(fillups) + "\n";
							string = string + "Miles: " + QString::number(miles,'f',2) + "\n";
							string = string + "Gallons: " + QString::number(gallons,'f',2) + "\n";
							string = string + "Cost: "+currencySymbol + QString::number(cost,'f',2) + "\n";
							string = string + "MPG: " + QString::number(mpg,'f',3);
							
							t.append(string);
							if (whichcolor == 0)
								rowcolor.append(one);
							else
								rowcolor.append(two);
						}
					} //end query
					else
						QMessageBox::critical(this,tr("Error!"),query.lastError().text(),QMessageBox::Ok,0,0);
				}//end of the years
				//add row to the model
				model->setModelData(t,rowcolor);
				//switch color
				if (whichcolor == 0)
					whichcolor = 1;
				else if (whichcolor == 1)
					whichcolor = 0;
				
			}//end of the months

			//set the table model
			monthlyTable->setModel(model);
			
			//resize the rows and columns to match the data
			monthlyTable->resizeColumnsToContents();
			monthlyTable->resizeRowsToContents();

			//had to go back, the columns weren't quite big enough with the function above 
/*			for(int i=0;i<model->columnCount();i++)
			{
				monthlyTable->setColumnWidth(i,120);
			}*/
		}//end big query is ok
	}//end connection is open
}//end monthyStats

//slot to refresh whichever tab was clicked on
void Statistics::refreshTables()
{
	if(statsBase.tabWidget->tabText(statsBase.tabWidget->currentIndex()) == "Lifetime")
		lifetimeStats();
	if(statsBase.tabWidget->tabText(statsBase.tabWidget->currentIndex()) == "Yearly")
		yearlyStats();
	if(statsBase.tabWidget->tabText(statsBase.tabWidget->currentIndex()) == "Monthly")
		monthlyStats();
}
